@@ -5,23 +5,14 @@
 #define XXXXXXX    0
 #define USE_PENALTY_EX  1
 
-
 #if 1
-static void match_calc( float *match, char **s1, char **s2, int i1, int lgth2 ) 
+static void match_calc_mtx( double **mtx, float *match, char **s1, char **s2, int i1, int lgth2 ) 
 {
 	char *seq2 = s2[0];
-	int *intptr = amino_dis[(int)s1[0][i1]];
+	double *doubleptr = mtx[(int)s1[0][i1]];
 
 	while( lgth2-- )
-		*match++ = intptr[(int)*seq2++];
-}
-static void match_calc_mtx( int mtx[0x80][0x80], float *match, char **s1, char **s2, int i1, int lgth2 ) 
-{
-	char *seq2 = s2[0];
-	int *intptr = mtx[(int)s1[0][i1]];
-
-	while( lgth2-- )
-		*match++ = intptr[(int)*seq2++];
+		*match++ = doubleptr[(int)*seq2++];
 }
 #else
 static void match_calc( float *match, char **s1, char **s2, int i1, int lgth2 )
@@ -37,7 +28,8 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 						char **seq1, char **seq2, 
                         char **mseq1, char **mseq2, 
                         int **ijp,
-						int tailgp )
+						int tailgp,
+						int *warpis, int *warpjs, int warpbase )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k, limk;
 //	char gap[] = "-";
@@ -64,6 +56,7 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
         ijp[0][j] = -( j + 1 );
     }
 
+//	if( tailgp == 1 || ijp[lgth1][lgth2] >= warpbase )
 	if( tailgp == 1 )
 		;
 	else
@@ -97,11 +90,18 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 	*mseq2[0] = 0;
 
 
+
 	iin = lgth1; jin = lgth2;
 	limk = lgth1+lgth2 + 1;
 	for( k=0; k<limk; k++ ) 
 	{
-		if( ijp[iin][jin] < 0 ) 
+		if( ijp[iin][jin] >= warpbase )
+		{
+//			fprintf( stderr, "WARP!\n" );
+			ifi = warpis[ijp[iin][jin]-warpbase]; 
+			jfi = warpjs[ijp[iin][jin]-warpbase];
+		}
+		else if( ijp[iin][jin] < 0 ) 
 		{
 			ifi = iin-1; jfi = jin+ijp[iin][jin];
 		}
@@ -113,19 +113,41 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 		{
 			ifi = iin-1; jfi = jin-1;
 		}
-		l = iin - ifi;
-		while( --l ) 
+
+		if( ifi == -warpbase && jfi == -warpbase )
 		{
-			*--mseq1[0] = seq1[0][ifi+l];
-			*--mseq2[0] = *gap;
-			k++;
+			l = iin;
+			while( --l >= 0 ) 
+			{
+				*--mseq1[0] = seq1[0][l];
+				*--mseq2[0] = *gap;
+				k++;
+			}
+			l= jin;
+			while( --l >= 0 )
+			{
+				*--mseq1[0] = *gap;
+				*--mseq2[0] = seq2[0][l];
+				k++;
+			}
+			break;
 		}
-		l= jin - jfi;
-		while( --l )
+		else
 		{
-			*--mseq1[0] = *gap;
-			*--mseq2[0] = seq2[0][jfi+l];
-			k++;
+			l = iin - ifi;
+			while( --l > 0 ) 
+			{
+				*--mseq1[0] = seq1[0][ifi+l];
+				*--mseq2[0] = *gap;
+				k++;
+			}
+			l= jin - jfi;
+			while( --l > 0 )
+			{
+				*--mseq1[0] = *gap;
+				*--mseq2[0] = seq2[0][jfi+l];
+				k++;
+			}
 		}
 		if( iin <= 0 || jin <= 0 ) break;
 		*--mseq1[0] = seq1[0][ifi];
@@ -133,22 +155,27 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 		k++;
 		iin = ifi; jin = jfi;
 	}
+
+//	fprintf( stderr, "%s\n", mseq1[0] );
+//	fprintf( stderr, "%s\n", mseq2[0] );
 	return( 0.0 );
 }
 
 
-float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp )
-/* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
+float G__align11( double **n_dynamicmtx, char **seq1, char **seq2, int alloclen, int headgp, int tailgp )
 {
 //	int k;
 	register int i, j;
 	int lasti;                      /* outgap == 0 -> lgth1, outgap == 1 -> lgth1+1 */
+	int lastj;
 	int lgth1, lgth2;
 	int resultlen;
 	float wm;   /* int ?????? */
 	float g;
 	float *currentw, *previousw;
 	float fpenalty = (float)penalty;
+	float fpenalty_shift = (float)penalty_shift;
+	float fpenalty_tmp;
 #if USE_PENALTY_EX
 	float fpenalty_ex = (float)penalty_ex;
 #endif
@@ -174,6 +201,22 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 	static TLS int **intwork = NULL;
 	static TLS float **floatwork = NULL;
 	static TLS int orlgth1 = 0, orlgth2 = 0;
+	static TLS double **amino_dynamicmtx = NULL; // ??
+
+	int *warpis = NULL;
+	int *warpjs = NULL;
+	int *warpi = NULL;
+	int *warpj = NULL;
+	int *prevwarpi = NULL;
+	int *prevwarpj = NULL;
+	float *wmrecords = NULL;
+	float *prevwmrecords = NULL;
+	int warpn = 0;
+	int warpbase;
+	float curm = 0.0;
+	float *wmrecordspt, *wmrecords1pt, *prevwmrecordspt;
+	int *warpipt, *warpjpt;
+
 
 	if( seq1 == NULL )
 	{
@@ -198,13 +241,46 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 
 			if( floatwork ) FreeFloatMtx( floatwork ); floatwork = NULL;
 			if( intwork ) FreeIntMtx( intwork ); intwork = NULL;
+
+			if( amino_dynamicmtx ) FreeDoubleMtx( amino_dynamicmtx ); amino_dynamicmtx = NULL;
 		}
+		orlgth1 = 0;
+		orlgth2 = 0;
 		return( 0.0 );
 	}
 
+
+
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
 
+	warpbase = lgth1 + lgth2;
+	warpis = NULL;
+	warpjs = NULL;
+	warpn = 0;
+	if( trywarp )
+	{
+//		fprintf( stderr, "IN G__align11\n" );
+		if( headgp == 0 || tailgp == 0 )
+		{
+			fprintf( stderr, "At present, headgp and tailgp must be 1.\n" );
+			exit( 1 );
+		}
+
+		wmrecords = AllocateFloatVec( lgth2+1 );
+		warpi = AllocateIntVec( lgth2+1 );
+		warpj = AllocateIntVec( lgth2+1 );
+		prevwmrecords = AllocateFloatVec( lgth2+1 );
+		prevwarpi = AllocateIntVec( lgth2+1 );
+		prevwarpj = AllocateIntVec( lgth2+1 );
+		for( i=0; i<lgth2+1; i++ ) prevwmrecords[i] = 0.0;
+		for( i=0; i<lgth2+1; i++ ) wmrecords[i] = 0.0;
+		for( i=0; i<lgth2+1; i++ ) prevwarpi[i] = -warpbase;
+		for( i=0; i<lgth2+1; i++ ) prevwarpj[i] = -warpbase;
+		for( i=0; i<lgth2+1; i++ ) warpi[i] = -warpbase;
+		for( i=0; i<lgth2+1; i++ ) warpj[i] = -warpbase;
+	}
+
 #if 0
 	if( lgth1 <= 0 || lgth2 <= 0 )
 	{
@@ -265,6 +341,7 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 
 			FreeFloatMtx( floatwork );
 			FreeIntMtx( intwork );
+			FreeDoubleMtx( amino_dynamicmtx );
 		}
 
 		ll1 = MAX( (int)(1.3*lgth1), orlgth1 ) + 100;
@@ -289,6 +366,7 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 
 		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
+		amino_dynamicmtx = AllocateDoubleMtx( 0x80, 0x80 );
 
 #if DEBUG
 		fprintf( stderr, "succeeded\n" );
@@ -297,6 +375,8 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 		orlgth1 = ll1 - 100;
 		orlgth2 = ll2 - 100;
 	}
+    for( i=0; i<nalphabets; i++) for( j=0; j<nalphabets; j++ )
+		amino_dynamicmtx[(int)amino[i]][(int)amino[j]] = (double)n_dynamicmtx[i][j];
 
 
 	mseq1[0] = mseq[0];
@@ -340,10 +420,8 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 	previousw = w2;
 
 
-	match_calc( initverticalw, seq2, seq1, 0, lgth1 );
-
-
-	match_calc( currentw, seq1, seq2, 0, lgth2 );
+	match_calc_mtx( amino_dynamicmtx, initverticalw, seq2, seq1, 0, lgth1 );
+	match_calc_mtx( amino_dynamicmtx, currentw, seq1, seq2, 0, lgth2 );
 
 	if( headgp == 1 )
 	{
@@ -368,6 +446,7 @@ float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp
 		lastverticalw[0] = currentw[lgth2-1]; // lgth2==0 no toki error
 
 	if( tailgp ) lasti = lgth1+1; else lasti = lgth1;
+	lastj = lgth2+1;
 
 #if XXXXXXX
 fprintf( stderr, "currentw = \n" );
@@ -392,7 +471,7 @@ fprintf( stderr, "\n" );
 
 		previousw[0] = initverticalw[i-1];
 
-		match_calc( currentw, seq1, seq2, i, lgth2 );
+		match_calc_mtx( amino_dynamicmtx, currentw, seq1, seq2, i, lgth2 );
 #if XXXXXXX
 fprintf( stderr, "\n" );
 fprintf( stderr, "i=%d\n", i );
@@ -422,8 +501,18 @@ fprintf( stderr, "\n" );
 		prept = previousw;
 		curpt = currentw + 1;
 		mpjpt = mp + 1;
-		for( j=1; j<lgth2+1; j++ )
+		if( trywarp )
 		{
+			prevwmrecordspt = prevwmrecords;
+			wmrecordspt = wmrecords+1;
+			wmrecords1pt = wmrecords;
+			warpipt = warpi + 1;
+			warpjpt = warpj + 1;
+		}
+
+		for( j=1; j<lastj; j++ )
+		{
+
 			wm = *prept;
 			*ijppt = 0;
 
@@ -463,10 +552,85 @@ fprintf( stderr, "\n" );
 #if USE_PENALTY_EX
 			m[j] += fpenalty_ex;
 #endif
+#if 1
+			if( trywarp )
+			{
+				fpenalty_tmp = fpenalty_shift + fpenalty_ex * ( i - prevwarpi[j-1] + j - prevwarpj[j-1] );
+//				fprintf( stderr, "fpenalty_shift = %f\n", fpenalty_tmp );
 
-#if 0
-			fprintf( stderr, "%5.0f ", wm );
+//				fprintf( stderr, "\n\n\nwarp to %c-%c (%d-%d) from %c-%c (%d-%d) ? prevwmrecords[%d] = %f + %f <- wm = %f\n", seq1[0][prevwarpi[j-1]], seq2[0][prevwarpj[j-1]], prevwarpi[j-1], prevwarpj[j-1], seq1[0][i], seq2[0][j], i, j, j, prevwmrecords[j-1], fpenalty_tmp, wm );
+//				if( (g=prevwmrecords[j-1] + fpenalty_shift )> wm )
+				if( ( g=*prevwmrecordspt++ + fpenalty_tmp )> wm ) // naka ha osokute kamawanai
+				{
+//					fprintf( stderr, "Yes! Warp!! from %d-%d (%c-%c) to  %d-%d (%c-%c) fpenalty_tmp = %f! warpn = %d\n", i, j, seq1[0][i], seq2[0][j-1], prevwarpi[j-1], prevwarpj[j-1],seq1[0][prevwarpi[j-1]], seq2[0][prevwarpj[j-1]], fpenalty_tmp, warpn );
+					if( warpn && prevwarpi[j-1] == warpis[warpn-1] && prevwarpj[j-1] == warpjs[warpn-1] )
+					{
+						*ijppt = warpbase + warpn - 1;
+					}
+					else
+					{
+						*ijppt = warpbase + warpn;
+						warpis = realloc( warpis, sizeof(int) * ( warpn+1 ) );
+						warpjs = realloc( warpjs, sizeof(int) * ( warpn+1 ) );
+						warpis[warpn] = prevwarpi[j-1];
+						warpjs[warpn] = prevwarpj[j-1];
+						warpn++;
+					}
+					wm = g;
+				}
+				else
+				{
+				}
+
+				curm = *curpt + wm;
+	
+//				fprintf( stderr, "###### curm = %f at %c-%c, i=%d, j=%d\n", curm, seq1[0][i], seq2[0][j], i, j ); 
+	
+//				fprintf( stderr, "copy from i, j-1? %f > %f?\n", wmrecords[j-1], curm );
+//				if( wmrecords[j-1] > wmrecords[j] )
+				if( *wmrecords1pt > *wmrecordspt )
+				{
+//					fprintf( stderr, "yes\n" );
+//					wmrecords[j] = wmrecords[j-1];
+					*wmrecordspt = *wmrecords1pt;
+//					warpi[j] = warpi[j-1];
+//					warpj[j] = warpj[j-1];
+					*warpipt  = *(warpipt-1);
+					*warpjpt  = *(warpjpt-1);
+//					fprintf( stderr, "warpi[j]=%d, warpj[j]=%d wmrecords[j] = %f\n", warpi[j], warpj[j], wmrecords[j] );
+				}
+//				else
+//				{
+//					fprintf( stderr, "no\n" );
+//				}
+	
+//				fprintf( stderr, " curm = %f at %c-%c\n", curm, seq1[0][i], seq2[0][j] ); 
+//				fprintf( stderr, " wmrecords[%d] = %f\n", j, wmrecords[j] ); 
+//				fprintf( stderr, "replace?\n" );
+	
+//				if( curm > wmrecords[j] )
+				if( curm > *wmrecordspt )
+				{
+//					fprintf( stderr, "yes at %d-%d (%c-%c), replaced warp: warpi[j]=%d, warpj[j]=%d warpn=%d, wmrecords[j] = %f -> %f\n", i, j, seq1[0][i], seq2[0][j], i, j, warpn, wmrecords[j], curm );
+//					wmrecords[j] = curm;
+					*wmrecordspt = curm;
+//					warpi[j] = i;
+//					warpj[j] = j;
+					*warpipt = i;
+					*warpjpt = j;
+				}
+//				else
+//				{
+//					fprintf( stderr, "No! warpi[j]=%d, warpj[j]=%d wmrecords[j] = %f\n", warpi[j], warpj[j], wmrecords[j] );
+//				}
+//				fprintf( stderr, "%d-%d (%c-%c) curm = %5.0f, wmrecords[j]=%f\n", i, j, seq1[0][i], seq2[0][j], curm, wmrecords[j] );
+				wmrecordspt++;
+				wmrecords1pt++;
+				warpipt++;
+				warpjpt++;
+			}
 #endif
+
 			*curpt++ += wm;
 			ijppt++;
 			mjpt++;
@@ -474,9 +638,31 @@ fprintf( stderr, "\n" );
 			mpjpt++;
 		}
 		lastverticalw[i] = currentw[lgth2-1]; // lgth2==0 no toki error
+
+		if( trywarp )
+		{
+			fltncpy( prevwmrecords, wmrecords, lastj );
+			intncpy( prevwarpi, warpi, lastj );
+			intncpy( prevwarpj, warpj, lastj );
+		}
 	}
 
-	Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, tailgp );
+	if( trywarp )
+	{
+//		fprintf( stderr, "\nwm = %f\n", wm );
+//		fprintf( stderr, "warpn = %d\n", warpn );
+		free( wmrecords );
+		free( prevwmrecords );
+		free( warpi );
+		free( warpj );
+		free( prevwarpi );
+		free( prevwarpj );
+	}
+
+	Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, tailgp, warpis, warpjs, warpbase );
+	if( warpis ) free( warpis );
+	if( warpjs ) free( warpjs );
+
 
 	resultlen = strlen( mseq1[0] );
 	if( alloclen < resultlen || resultlen > N )
@@ -498,8 +684,8 @@ fprintf( stderr, "\n" );
 	return( wm );
 }
 
-float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, char **seq1, char **seq2, int alloclen )
-/* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
+float G__align11_noalign( double **n_dynamicmtx, int penal, int penal_ex, char **seq1, char **seq2, int alloclen )
+/* warp mitaiou */
 {
 //	int k;
 	register int i, j;
@@ -526,6 +712,7 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 	static TLS int **intwork;
 	static TLS float **floatwork;
 	static TLS int orlgth1 = 0, orlgth2 = 0;
+	static TLS double **amino_dynamicmtx;
 
 	if( seq1 == NULL )
 	{
@@ -543,10 +730,12 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 
 			FreeFloatMtx( floatwork );
 			FreeIntMtx( intwork );
+			FreeDoubleMtx( amino_dynamicmtx );
 		}
 		return( 0.0 );
 	}
 
+
 	wm = 0.0;
 
 
@@ -582,6 +771,8 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 
 			FreeFloatMtx( floatwork );
 			FreeIntMtx( intwork );
+
+			FreeDoubleMtx( amino_dynamicmtx );
 		}
 
 		ll1 = MAX( (int)(1.3*lgth1), orlgth1 ) + 100;
@@ -605,6 +796,8 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 
+
+		amino_dynamicmtx = AllocateDoubleMtx( 0x80, 0x80 );
 #if DEBUG
 		fprintf( stderr, "succeeded\n" );
 #endif
@@ -614,6 +807,8 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 	}
 
 
+    for( i=0; i<nalphabets; i++) for( j=0; j<nalphabets; j++ )
+		amino_dynamicmtx[(int)amino[i]][(int)amino[j]] = (double)n_dynamicmtx[i][j];
 
 
 
@@ -627,10 +822,10 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 	previousw = w2;
 
 
-	match_calc_mtx( scoremtx, initverticalw, seq2, seq1, 0, lgth1 );
+	match_calc_mtx( amino_dynamicmtx, initverticalw, seq2, seq1, 0, lgth1 );
 
 
-	match_calc_mtx( scoremtx, currentw, seq1, seq2, 0, lgth2 );
+	match_calc_mtx( amino_dynamicmtx, currentw, seq1, seq2, 0, lgth2 );
 
 	if( 1 ) // tsuneni outgap-1
 	{
@@ -679,7 +874,7 @@ fprintf( stderr, "\n" );
 
 		previousw[0] = initverticalw[i-1];
 
-		match_calc_mtx( scoremtx, currentw, seq1, seq2, i, lgth2 );
+		match_calc_mtx( amino_dynamicmtx, currentw, seq1, seq2, i, lgth2 );
 #if XXXXXXX
 fprintf( stderr, "\n" );
 fprintf( stderr, "i=%d\n", i );
