@@ -10,14 +10,13 @@
 static TLS int localstop; // 060910
 
 #if 1
-static void match_calc( float *match, char **s1, char **s2, int i1, int lgth2 ) 
+static void match_calc_mtx( double **mtx, float *match, char **s1, char **s2, int i1, int lgth2 ) 
 {
 	char *seq2 = s2[0];
-	int *intptr;
+	double *doubleptr = mtx[(int)s1[0][i1]];
 
-	intptr = amino_dis[(int)s1[0][i1]];
 	while( lgth2-- )
-		*match++ = intptr[(int)*seq2++];
+		*match++ = doubleptr[(int)*seq2++];
 }
 #else
 static void match_calc( float *match, char **s1, char **s2, int i1, int lgth2 )
@@ -92,7 +91,8 @@ static void match_calc_bk( float *match, float **cpmx1, float **cpmx2, int i1, i
 static float Ltracking( float *lasthorizontalw, float *lastverticalw, 
 						char **seq1, char **seq2, 
                         char **mseq1, char **mseq2, 
-                        int **ijp, int *off1pt, int *off2pt, int endi, int endj )
+                        int **ijp, int *off1pt, int *off2pt, int endi, int endj,
+						int *warpis, int *warpjs, int warpbase )
 {
 	int i, j, l, iin, jin, lgth1, lgth2, k, limk;
 	int ifi=0, jfi=0; // by D.Mathog, a guess
@@ -126,7 +126,13 @@ static float Ltracking( float *lasthorizontalw, float *lastverticalw,
 	limk = lgth1+lgth2;
 	for( k=0; k<=limk; k++ ) 
 	{
-		if( ijp[iin][jin] < 0 ) 
+		if( ijp[iin][jin] >= warpbase )
+		{
+//			fprintf( stderr, "WARP!\n" );
+			ifi = warpis[ijp[iin][jin]-warpbase];
+			jfi = warpjs[ijp[iin][jin]-warpbase];
+		}
+		else if( ijp[iin][jin] < 0 ) 
 		{
 			ifi = iin-1; jfi = jin+ijp[iin][jin];
 		}
@@ -138,21 +144,47 @@ static float Ltracking( float *lasthorizontalw, float *lastverticalw,
 		{
 			ifi = iin-1; jfi = jin-1;
 		}
-		l = iin - ifi;
-		while( --l ) 
+
+
+#if 1 // sentou de warp?
+		if( ifi == -warpbase && jfi == -warpbase )
 		{
-			*--mseq1[0] = seq1[0][ifi+l];
-			*--mseq2[0] = *gap;
-			k++;
+			l = iin;
+			while( --l >= 0 ) 
+			{
+				*--mseq1[0] = seq1[0][l];
+				*--mseq2[0] = *gap;
+				k++;
+			}
+			l= jin;
+			while( --l >= 0 )
+			{
+				*--mseq1[0] = *gap;
+				*--mseq2[0] = seq2[0][l];
+				k++;
+			}
+			break;
 		}
-		l= jin - jfi;
-		while( --l )
+		else
+#endif
 		{
-			*--mseq1[0] = *gap;
-			*--mseq2[0] = seq2[0][jfi+l];
-			k++;
+			l = iin - ifi;
+			while( --l > 0 ) 
+			{
+				*--mseq1[0] = seq1[0][ifi+l];
+				*--mseq2[0] = *gap;
+				k++;
+			}
+			l= jin - jfi;
+			while( --l > 0 )
+			{
+				*--mseq1[0] = *gap;
+				*--mseq2[0] = seq2[0][jfi+l];
+				k++;
+			}
 		}
 
+
 		if( iin <= 0 || jin <= 0 ) break;
 		*--mseq1[0] = seq1[0][ifi];
 		*--mseq2[0] = seq2[0][jfi];
@@ -164,13 +196,16 @@ static float Ltracking( float *lasthorizontalw, float *lastverticalw,
 	if( jfi == -1 ) *off2pt = 0; else *off2pt = jfi;
 
 //	fprintf( stderr, "ifn = %d, jfn = %d\n", ifi, jfi );
+//	fprintf( stderr, "\n" );
+//	fprintf( stderr, "%s\n", mseq1[0] );
+//	fprintf( stderr, "%s\n", mseq2[0] );
 
 
 	return( 0.0 );
 }
 
 
-float L__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2pt )
+float L__align11( double **n_dynamicmtx, float scoreoffset, char **seq1, char **seq2, int alloclen, int *off1pt, int *off2pt )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
@@ -200,15 +235,34 @@ float L__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2
 //	static TLS int **intwork;
 //	static TLS float **floatwork;
 	static TLS int orlgth1 = 0, orlgth2 = 0;
+	static TLS double **amino_dynamicmtx = NULL; // ??
 	float maxwm;
 	int endali = 0, endalj = 0; // by D.Mathog, a guess
 //	int endali, endalj;
-	float localthr = -offset;
-	float localthr2 = -offset;
-//	float localthr = 100;
-//	float localthr2 = 100;
+	float localthr = -offset + scoreoffset * 600; // 2013/12/13
+	float localthr2 = -offset + scoreoffset * 600; // 2013/12/13
+//	float localthr = -offset;
+//	float localthr2 = -offset;
 	float fpenalty = (float)penalty;
 	float fpenalty_ex = (float)penalty_ex;
+	float fpenalty_shift = (float)penalty_shift;
+	float fpenalty_tmp; // atode kesu
+
+	int *warpis = NULL;
+	int *warpjs = NULL;
+	int *warpi = NULL;
+	int *warpj = NULL;
+	int *prevwarpi = NULL;
+	int *prevwarpj = NULL;
+	float *wmrecords = NULL;
+	float *prevwmrecords = NULL;
+	int warpn = 0;
+	int warpbase;
+	float curm = 0.0;
+	float *wmrecordspt, *wmrecords1pt, *prevwmrecordspt;
+	int *warpipt, *warpjpt;
+
+
 
 	if( seq1 == NULL )
 	{
@@ -228,6 +282,7 @@ float L__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2
 			FreeIntVec( mp );
 
 			FreeCharMtx( mseq );
+			if( amino_dynamicmtx ) FreeDoubleMtx( amino_dynamicmtx ); amino_dynamicmtx = NULL;
 
 		}
 		return( 0.0 );
@@ -244,6 +299,28 @@ float L__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
 
+
+	warpbase = lgth1 + lgth2;
+	warpis = NULL;
+	warpjs = NULL;
+	warpn = 0;
+	if( trywarp )
+	{
+		wmrecords = AllocateFloatVec( lgth2+1 );
+		warpi = AllocateIntVec( lgth2+1 );
+		warpj = AllocateIntVec( lgth2+1 );
+		prevwmrecords = AllocateFloatVec( lgth2+1 );
+		prevwarpi = AllocateIntVec( lgth2+1 );
+		prevwarpj = AllocateIntVec( lgth2+1 );
+		for( i=0; i<lgth2+1; i++ ) prevwmrecords[i] = 0.0;
+		for( i=0; i<lgth2+1; i++ ) wmrecords[i] = 0.0;
+		for( i=0; i<lgth2+1; i++ ) prevwarpi[i] = -warpbase;
+		for( i=0; i<lgth2+1; i++ ) prevwarpj[i] = -warpbase;
+		for( i=0; i<lgth2+1; i++ ) warpi[i] = -warpbase;
+		for( i=0; i<lgth2+1; i++ ) warpj[i] = -warpbase;
+	}
+
+
 	if( lgth1 > orlgth1 || lgth2 > orlgth2 )
 	{
 		int ll1, ll2;
@@ -260,7 +337,7 @@ float L__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2
 			FreeIntVec( mp );
 
 			FreeCharMtx( mseq );
-
+			if( amino_dynamicmtx ) FreeDoubleMtx( amino_dynamicmtx ); amino_dynamicmtx = NULL;
 
 
 //			FreeFloatMtx( floatwork );
@@ -293,11 +370,14 @@ float L__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2
 #if DEBUG
 		fprintf( stderr, "succeeded\n" );
 #endif
-
+		amino_dynamicmtx = AllocateDoubleMtx( 0x80, 0x80 );
 		orlgth1 = ll1 - 100;
 		orlgth2 = ll2 - 100;
 	}
 
+    for( i=0; i<nalphabets; i++) for( j=0; j<nalphabets; j++ )
+		amino_dynamicmtx[(int)amino[i]][(int)amino[j]] = (double)n_dynamicmtx[i][j];
+
 
 	mseq1[0] = mseq[0];
 	mseq2[0] = mseq[1];
@@ -339,9 +419,9 @@ float L__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2
 	currentw = w1;
 	previousw = w2;
 
-	match_calc( initverticalw, seq2, seq1, 0, lgth1 );
+	match_calc_mtx( amino_dynamicmtx, initverticalw, seq2, seq1, 0, lgth1 );
 
-	match_calc( currentw, seq1, seq2, 0, lgth2 );
+	match_calc_mtx( amino_dynamicmtx, currentw, seq1, seq2, 0, lgth2 );
 
 
 	lasti = lgth2+1;
@@ -398,7 +478,7 @@ fprintf( stderr, "\n" );
 
 		previousw[0] = initverticalw[i-1];
 
-		match_calc( currentw, seq1, seq2, i, lgth2 );
+		match_calc_mtx( amino_dynamicmtx, currentw, seq1, seq2, i, lgth2 );
 #if DEBUG2
 		fprintf( stderr, "%c   ", seq1[0][i] );
 		fprintf( stderr, "%5.0f ", currentw[0] );
@@ -438,6 +518,15 @@ fprintf( stderr, "\n" );
 		curpt = currentw + 1;
 		mpjpt = mp + 1;
 		lastj = lgth2+1;
+
+		if( trywarp )
+		{
+			prevwmrecordspt = prevwmrecords;
+			wmrecordspt = wmrecords+1;
+			wmrecords1pt = wmrecords;
+			warpipt = warpi + 1;
+			warpjpt = warpj + 1;
+		}
 		for( j=1; j<lastj; j++ )
 		{
 			wm = *prept;
@@ -490,7 +579,7 @@ fprintf( stderr, "\n" );
 #if 1
 			if( wm < localthr )
 			{
-//				fprintf( stderr, "stop i=%d, j=%d, curpt=%f\n", i, j, *curpt );
+//				fprintf( stderr, "stop i=%d, j=%d, curpt=%f, localthr = %f\n", i, j, *curpt, localthr );
 				*ijppt = localstop;
 				wm = localthr2;
 			}
@@ -498,10 +587,86 @@ fprintf( stderr, "\n" );
 #if 0
 			fprintf( stderr, "%5.0f ", *curpt );
 #endif
-#if DEBUG2
-			fprintf( stderr, "%5.0f ", wm );
+#if 0
+			fprintf( stderr, "wm (%d,%d) = %5.0f\n", i, j, wm );
 //			fprintf( stderr, "%c-%c *ijppt = %d, localstop = %d\n", seq1[0][i], seq2[0][j], *ijppt, localstop );
 #endif
+			if( trywarp )
+			{
+				fpenalty_tmp = fpenalty_shift + fpenalty_ex * ( i - prevwarpi[j-1] + j - prevwarpj[j-1] );
+//				fprintf( stderr, "fpenalty_shift = %f\n", fpenalty_tmp );
+
+//				fprintf( stderr, "\n\n\nwarp to %c-%c (%d-%d) from %c-%c (%d-%d) ? prevwmrecords[%d] = %f + %f <- wm = %f\n", seq1[0][prevwarpi[j-1]], seq2[0][prevwarpj[j-1]], prevwarpi[j-1], prevwarpj[j-1], seq1[0][i], seq2[0][j], i, j, j, prevwmrecords[j-1], fpenalty_tmp, wm );
+//				if( (g=prevwmrecords[j-1] + fpenalty_shift )> wm )
+				if( ( g=*prevwmrecordspt++ + fpenalty_tmp )> wm ) // naka ha osokute kamawanai
+				{
+//					fprintf( stderr, "Yes! Warp!! from %d-%d (%c-%c) to  %d-%d (%c-%c) fpenalty_tmp = %f! warpn = %d\n", i, j, seq1[0][i], seq2[0][j-1], prevwarpi[j-1], prevwarpj[j-1],seq1[0][prevwarpi[j-1]], seq2[0][prevwarpj[j-1]], fpenalty_tmp, warpn );
+					if( warpn && prevwarpi[j-1] == warpis[warpn-1] && prevwarpj[j-1] == warpjs[warpn-1] )
+					{
+						*ijppt = warpbase + warpn - 1;
+					}
+					else
+					{
+						*ijppt = warpbase + warpn;
+						warpis = realloc( warpis, sizeof(int) * ( warpn+1 ) );
+						warpjs = realloc( warpjs, sizeof(int) * ( warpn+1 ) );
+						warpis[warpn] = prevwarpi[j-1];
+						warpjs[warpn] = prevwarpj[j-1];
+						warpn++;
+					}
+					wm = g;
+				}
+				else
+				{
+				}
+
+				curm = *curpt + wm;
+	
+//				fprintf( stderr, "###### curm = %f at %c-%c, i=%d, j=%d\n", curm, seq1[0][i], seq2[0][j], i, j ); 
+	
+//				fprintf( stderr, "copy from i, j-1? %f > %f?\n", wmrecords[j-1], curm );
+//				if( wmrecords[j-1] > wmrecords[j] )
+				if( *wmrecords1pt > *wmrecordspt )
+				{
+//					fprintf( stderr, "yes\n" );
+//					wmrecords[j] = wmrecords[j-1];
+					*wmrecordspt = *wmrecords1pt;
+//					warpi[j] = warpi[j-1];
+//					warpj[j] = warpj[j-1];
+					*warpipt  = *(warpipt-1);
+					*warpjpt  = *(warpjpt-1);
+//					fprintf( stderr, "warpi[j]=%d, warpj[j]=%d wmrecords[j] = %f\n", warpi[j], warpj[j], wmrecords[j] );
+				}
+//				else
+//				{
+//					fprintf( stderr, "no\n" );
+//				}
+	
+//				fprintf( stderr, " curm = %f at %c-%c\n", curm, seq1[0][i], seq2[0][j] ); 
+//				fprintf( stderr, " wmrecords[%d] = %f\n", j, wmrecords[j] ); 
+//				fprintf( stderr, "replace?\n" );
+	
+//				if( curm > wmrecords[j] )
+				if( curm > *wmrecordspt )
+				{
+//					fprintf( stderr, "yes at %d-%d (%c-%c), replaced warp: warpi[j]=%d, warpj[j]=%d warpn=%d, wmrecords[j] = %f -> %f\n", i, j, seq1[0][i], seq2[0][j], i, j, warpn, wmrecords[j], curm );
+//					wmrecords[j] = curm;
+					*wmrecordspt = curm;
+//					warpi[j] = i;
+//					warpj[j] = j;
+					*warpipt = i;
+					*warpjpt = j;
+				}
+//				else
+//				{
+//					fprintf( stderr, "No! warpi[j]=%d, warpj[j]=%d wmrecords[j] = %f\n", warpi[j], warpj[j], wmrecords[j] );
+//				}
+//				fprintf( stderr, "%d-%d (%c-%c) curm = %5.0f, wmrecords[j]=%f\n", i, j, seq1[0][i], seq2[0][j], curm, wmrecords[j] );
+				wmrecordspt++;
+				wmrecords1pt++;
+				warpipt++;
+				warpjpt++;
+			}
 
 			*curpt++ += wm;
 			ijppt++;
@@ -514,8 +679,25 @@ fprintf( stderr, "\n" );
 #endif
 
 		lastverticalw[i] = currentw[lgth2-1];
-	}
+		if( trywarp )
+		{
+			fltncpy( prevwmrecords, wmrecords, lastj );
+			intncpy( prevwarpi, warpi, lastj );
+			intncpy( prevwarpj, warpj, lastj );
+		}
 
+	}
+//	fprintf( stderr, "\nwm = %f\n", wm );
+	if( trywarp )
+	{
+//		if( warpn ) fprintf( stderr, "warpn = %d\n", warpn );
+		free( wmrecords );
+		free( prevwmrecords );
+		free( warpi );
+		free( warpj );
+		free( prevwarpi );
+		free( prevwarpj );
+	}
 
 #if 0
 	fprintf( stderr, "maxwm = %f\n", maxwm );
@@ -532,7 +714,9 @@ fprintf( stderr, "\n" );
 		return( 0.0 );
 	}
 		
-	Ltracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, off1pt, off2pt, endali, endalj );
+	Ltracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, off1pt, off2pt, endali, endalj, warpis, warpjs, warpbase );
+	if( warpis ) free( warpis );
+	if( warpjs ) free( warpjs );
 
 
 	resultlen = strlen( mseq1[0] );
@@ -559,8 +743,8 @@ fprintf( stderr, "\n" );
 }
 
 
-float L__align11_noalign( char **seq1, char **seq2 )
-/* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
+float L__align11_noalign( double **n_dynamicmtx, char **seq1, char **seq2 )
+// warp mitaiou
 {
 //	int k;
 	int i, j;
@@ -589,6 +773,7 @@ float L__align11_noalign( char **seq1, char **seq2 )
 //	static TLS int **intwork;
 //	static TLS float **floatwork;
 	static TLS int orlgth1 = 0, orlgth2 = 0;
+	static TLS double **amino_dynamicmtx = NULL; // ??
 	float maxwm;
 //	int endali = 0, endalj = 0; // by D.Mathog, a guess
 //	int endali, endalj;
@@ -617,6 +802,7 @@ float L__align11_noalign( char **seq1, char **seq2 )
 //			FreeIntVec( mp );
 
 //			FreeCharMtx( mseq );
+			if( amino_dynamicmtx ) FreeDoubleMtx( amino_dynamicmtx ); amino_dynamicmtx = NULL;
 
 		}
 		return( 0.0 );
@@ -654,6 +840,7 @@ float L__align11_noalign( char **seq1, char **seq2 )
 
 //			FreeFloatMtx( floatwork );
 //			FreeIntMtx( intwork );
+			if( amino_dynamicmtx ) FreeDoubleMtx( amino_dynamicmtx ); amino_dynamicmtx = NULL;
 		}
 
 		ll1 = MAX( (int)(1.3*lgth1), orlgth1 ) + 100;
@@ -682,11 +869,15 @@ float L__align11_noalign( char **seq1, char **seq2 )
 #if DEBUG
 		fprintf( stderr, "succeeded\n" );
 #endif
-
+		amino_dynamicmtx = AllocateDoubleMtx( 0x80, 0x80 );
 		orlgth1 = ll1 - 100;
 		orlgth2 = ll2 - 100;
 	}
 
+    for( i=0; i<nalphabets; i++) for( j=0; j<nalphabets; j++ )
+		amino_dynamicmtx[(int)amino[i]][(int)amino[j]] = (double)n_dynamicmtx[i][j];
+
+
 
 //	mseq1[0] = mseq[0];
 //	mseq2[0] = mseq[1];
@@ -728,9 +919,9 @@ float L__align11_noalign( char **seq1, char **seq2 )
 	currentw = w1;
 	previousw = w2;
 
-	match_calc( initverticalw, seq2, seq1, 0, lgth1 );
+	match_calc_mtx( amino_dynamicmtx, initverticalw, seq2, seq1, 0, lgth1 );
 
-	match_calc( currentw, seq1, seq2, 0, lgth2 );
+	match_calc_mtx( amino_dynamicmtx, currentw, seq1, seq2, 0, lgth2 );
 
 
 	lasti = lgth2+1;
@@ -788,7 +979,7 @@ fprintf( stderr, "\n" );
 
 		previousw[0] = initverticalw[i-1];
 
-		match_calc( currentw, seq1, seq2, i, lgth2 );
+		match_calc_mtx( amino_dynamicmtx, currentw, seq1, seq2, i, lgth2 );
 #if DEBUG2
 		fprintf( stderr, "%c   ", seq1[0][i] );
 		fprintf( stderr, "%5.0f ", currentw[0] );
