@@ -825,6 +825,7 @@ void arguments( int argc, char *argv[] )
 	tbrweight = 3;
 	checkC = 0;
 	treemethod = 'X';
+	sueff_global = 0.1;
 	contin = 0;
 	scoremtx = 1;
 	kobetsubunkatsu = 0;
@@ -994,7 +995,10 @@ void arguments( int argc, char *argv[] )
 #endif
 				case 'X':
 					treemethod = 'X';
-					break;
+					sueff_global = atof( *++argv );
+					fprintf( stderr, "sueff_global = %f\n", sueff_global );
+					--argc;
+					goto nextoption;
 				case 'E':
 					treemethod = 'E';
 					break;
@@ -1720,7 +1724,7 @@ static void	*addsinglethread( void *arg )
 //			if( iadd == 0 )
 //			{
 //			}
-//			fixed_musclesupg_float_realloc_nobk_halfmtx( njobc, iscorec, topolc, lenc, depc, 0 );
+//			fixed_musclesupg_float_realloc_nobk_halfmtx( njobc, iscorec, topolc, lenc, depc, 0, 1 );
 			neighbor = addonetip( njobc, topolc, lenc, iscorec, topol, len, dep, treeout, addtree, iadd, name, alnleninnode, nogaplenjusttodecideaddhereornot );
 
 			if( noalign ) 
@@ -1766,13 +1770,13 @@ static void	*addsinglethread( void *arg )
 				addmem[1] = -1;
 				for( i=0; i<njobc-1; i++ )
 				{
-					if( samemember( topolc[i][0], addmem ) ) // arieru
+					if( samemembern( topolc[i][0], addmem, 1 ) ) // arieru
 					{
 //						fprintf( stderr, "HIT!\n" );
 						if( mergeoralign[i] != 'n' ) mergeoralign[i] = 'w';
 						else mergeoralign[i] = '1';
 					}
-					else if( samemember( topolc[i][1], addmem ) )
+					else if( samemembern( topolc[i][1], addmem, 1 ) )
 					{
 //						fprintf( stderr, "HIT!\n" );
 						if( mergeoralign[i] != 'n' ) mergeoralign[i] = 'w';
@@ -2993,9 +2997,9 @@ int main( int argc, char *argv[] )
 
 //	fprintf( stderr, "building a tree.." );
 	if( treeout )
-		fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( norg, iscoreo, topol, len, name, nlen, dep );
+		fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( norg, iscoreo, topol, len, name, nlen, dep, 1 );
 	else
-		fixed_musclesupg_float_realloc_nobk_halfmtx( norg, iscoreo, topol, len, dep, 0 );
+		fixed_musclesupg_float_realloc_nobk_halfmtx( norg, iscoreo, topol, len, dep, 0, 1 );
 //	fprintf( stderr, "done.\n" );
 
 	if( norg > 1 ) 
@@ -3281,10 +3285,6 @@ int main( int argc, char *argv[] )
 	fprintf( stderr, "\n" );
 #endif
 
-	for( i=0; i<lenfull+1; i++ )
-	{
-	}
-
 	for( i=0; i<lenfull+1; i++ )
 	{
 		if( realign[i].nnewres > 1 ) 
