@@ -137,6 +137,40 @@ static void calcfreq( int nseq, char **seq, double *datafreq )
 	for( i=0; i<nscoredalphabets; i++ ) datafreq[i] /= (double)total;
 }
 
+static void calcfreq_extended( int nseq, char **seq, double *datafreq )
+{
+	int i, j, l;
+	int aan;
+	double total;
+	for( i=0; i<nscoredalphabets; i++ )
+		datafreq[i] = 0.0;
+	total = 0.0;
+	for( i=0; i<nseq; i++ )
+	{
+		l = strlen( seq[i] );
+		for( j=0; j<l; j++ )
+		{
+			aan = amino_n[(int)seq[i][j]];
+			if( aan >= 0 && aan < nscoredalphabets && seq[i][j] != '-' )
+			{
+				datafreq[aan] += 1.0;
+				total += 1.0;
+			}
+		}
+	}
+	total = 0.0; for( i=0; i<nscoredalphabets; i++ ) total += datafreq[i];
+	for( i=0; i<nscoredalphabets; i++ ) datafreq[i] /= (double)total;
+//	for( i=0; i<nscoredalphabets; i++ ) if( datafreq[i] < 0.0001 ) datafreq[i] = 0.0001;
+
+	reporterr(       "datafreq = \n" );
+	for( i=0; i<nscoredalphabets; i++ )
+		reporterr(       "%f\n", datafreq[i] );
+
+	total = 0.0; for( i=0; i<nscoredalphabets; i++ ) total += datafreq[i];
+	reporterr(       "total = %f\n", total );
+	for( i=0; i<nscoredalphabets; i++ ) datafreq[i] /= (double)total;
+}
+
 static void generatenuc1pam( double **pam1, int kimuraR, double *freq )
 {
 	int i, j;
@@ -594,11 +628,12 @@ void constants( int nseq, char **seq )
 		}
 		if( fmodel == 1 )
 		{
-			calcfreq( nseq, seq, datafreq );
+			calcfreq_extended( nseq, seq, datafreq );
 			freq1 = datafreq;
 		}
 		else
 			freq1 = freq;
+
 #if TEST
 		reporterr(       "raw scoreing matrix : \n" );
 		for( i=0; i<nalphabets; i++ )
@@ -686,6 +721,8 @@ void constants( int nseq, char **seq )
 
         if( disp )
         {
+			fprintf( stdout, "freq = \n" );
+			for( i=0; i<nalphabets; i++ ) fprintf( stdout, "%c %f\n", amino[i], freq1[i] );
             fprintf( stdout, " scoring matrix  \n" );
             for( i=0; i<nalphabets; i++ )
             {
