@@ -10,6 +10,7 @@ static int subalignmentoffset;
 
 static int intop;
 static int intree;
+static double autosubalignment;
 
 
 static void calcmaxdistclass( void )
@@ -76,6 +77,7 @@ void arguments( int argc, char *argv[] )
 	checkC = 0;
 	tbitr = 0;
 	treemethod = 'X';
+	sueff_global = 0.1;
 	scoremtx = 1;
 	dorp = NOTSPECIFIED;
 	ppenalty = NOTSPECIFIED;
@@ -97,6 +99,7 @@ void arguments( int argc, char *argv[] )
 	subalignmentoffset = 0;
 	legacygapcost = 0;
 	specificityconsideration = 0.0;
+	autosubalignment = 0.0;
 
 	while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -329,15 +332,25 @@ void arguments( int argc, char *argv[] )
 				case 'n' :
 					outnumber = 1;
 					break;
-				case 'X' :
+				case 'X':
 					treemethod = 'X';
-					break;
+					sueff_global = atof( *++argv );
+					fprintf( stderr, "sueff_global = %f\n", sueff_global );
+					--argc;
+					goto nextoption;
+#if 0
 				case 'E' :
 					treemethod = 'E';
 					break;
 				case 'q' :
 					treemethod = 'q';
 					break;
+#endif
+				case 'E':
+					autosubalignment = atof( *++argv );
+					fprintf( stderr, "autosubalignment = %f\n", autosubalignment );
+					--argc;
+					goto nextoption;
 				case 'z':
 					fftThreshold = myatoi( *++argv );
 					--argc;
@@ -694,6 +707,63 @@ exit( 1 );
 	printf( "utree = %d\n", utree );
 #endif
 
+	if( autosubalignment > 0.0 && subalignment == 0 )
+	{
+//		reporterr( "Computing skipthisbranch..\n" );
+		insubtable = AllocateIntVec( njob );
+		preservegaps = AllocateIntVec( njob );
+		subtable = calloc( 1, sizeof( char * ) );
+		subtable[0] = NULL; // for FreeIntMtx
+		for( i=0; i<njob; i++ ) insubtable[i] = 0;
+		for( i=0; i<njob; i++ ) preservegaps[i] = 0; // tsukawanaikamo
+		if( generatesubalignmentstable( njob, &subtable, &nsubalignments, &maxmem, topol, len, autosubalignment ) ) // subtable ha allocate sareru.
+		{
+			reporterr( "################################################################################################ \n" );
+			reporterr( "#\n" );
+			reporterr( "# WARNING: Iterative refinment was not done because you gave a large --fix value (%6.3f).\n", autosubalignment );
+			reporterr( "#\n" );
+			reporterr( "################################################################################################ \n" );
+			writePre( njob, name, nlen, seq_g, 1 );
+
+			FreeCharMtx( seq_g_bk );
+			FreeIntCub( topol );
+			FreeDoubleMtx( len );
+			FreeDoubleMtx( eff );
+			FreeIntMtx( skipthisbranch );
+			FreeIntMtx( subtable );
+			free( preservegaps );
+			free( insubtable );
+			SHOWVERSION;
+			return( 0 );
+		}
+//		subtable = AllocateIntMtx( nsubalignments, maxmem+1 );
+		fprintf( stderr, "nsubalignments = %d, maxmem = %d\n", nsubalignments, maxmem );
+		subalnpt = AllocateCharCub( nsubalignments, maxmem, 0 );
+#if 0
+		for( i=0; i<nsubalignments; i++ )
+		{
+			reporterr( "subalignment %d\n", i );
+			for( j=0; subtable[i][j]!=-1; j++ )
+			{
+				reporterr( "%5d", subtable[i][j] );
+			}
+			reporterr( "\n" );
+		}
+#endif
+#if 0 // wakaran
+		for( i=0; i<nsubalignments; i++ ) for( j=0; j<insubtable[i]; j++ )
+		{
+			if( subtable[i][j] < 0 )
+			{
+				fprintf( stderr, "Not supported in the iterative refinmenment mode.\n" );
+				fprintf( stderr, "Please use a positive number to represent a sequence.\n" );
+			}
+		}
+#endif
+//		reporterr( "done.\n" );
+	}
+
+
 	orderfp = fopen( "order", "w" );
 	if( !orderfp )
 	{
@@ -711,6 +781,7 @@ exit( 1 );
 	fclose( orderfp );
 
 
+
 	fprintf( stderr, "\n" );
 	if( ( !utree && kobetsubunkatsu ) || constraint || !bunkatsu )
 	{
@@ -743,14 +814,16 @@ exit( 1 );
 
 
 //--------------- kokokara ----
-	if( subalignment )
+	if( subalignment || autosubalignment )
 	{
 		for( i=0; i<nsubalignments; i++ )
 		{
-			fprintf( stderr, "Checking subalignment %d:\n", i+1 );
+			fprintf( stderr, "\nChecking subalignment %d:\n", i+1 );
 			alignmentlength = strlen( seq[subtable[i][0]] );
-//			for( j=0; subtable[i][j]!=-1; j++ )
-//				fprintf( stderr, " %d. %-30.30s\n", subtable[i][j]+1, name[subtable[i][j]]+1 );
+			for( j=0; subtable[i][j]!=-1; j++ )
+				fprintf( stderr, " %d ", subtable[i][j]+1 );
+//				fprintf( stderr, " ##### %d-%d. %-30.30s\n", i, subtable[i][j]+1, name[subtable[i][j]]+1 );
+			fprintf( stderr, "\n" );
 			for( j=0; subtable[i][j]!=-1; j++ )
 			{
 				if( subtable[i][j] >= njob )
@@ -786,10 +859,25 @@ exit( 1 );
 			}
 			for( j=0; j<njob-1; j++ )
 			{
+#if 0
+				int k;
+				reporterr( "####  STEP%d\n", j );
+				for( k=0; topol[j][0][k]!=-1; k++ ) reporterr( "%3d ", topol[j][0][k] );
+				reporterr( "len=%f\n", len[j][0] );
+				for( k=0; topol[j][1][k]!=-1; k++ ) reporterr( "%3d ", topol[j][1][k] );
+				reporterr( "len=%f\n", len[j][1] );
+				reporterr( "\n" );
+#endif
 				if( includemember( topol[j][0], subtable[i] ) && !samemember( topol[j][0], subtable[i] ) )
+				{
 					skipthisbranch[j][0] = 1;
+//					reporterr( "SKIP 0 !!!!!!\n" );
+				}
 				if( includemember( topol[j][1], subtable[i] ) && !samemember( topol[j][1], subtable[i] ) )
+				{
 					skipthisbranch[j][1] = 1;
+//					reporterr( "SKIP 1 !!!!!!\n" );
+				}
 			}
 			foundthebranch = 0;
 			for( j=0; j<njob-1; j++ )
