@@ -289,6 +289,7 @@ extern void getkyokaigap( char *g, char **s, int pos, int n );
 extern double *loadaamtx( void );
 extern float naivepairscore( int nseq1, int nseq2, char **seq1, char **seq2, double *eff1, double *eff2, int penal );
 extern float naivepairscore11( char *seq1, char *seq2, int penal );
+extern float naivepairscorefast( char *seq1, char *seq2, int *skip1, int *skip2, int penal );
 extern float naiveQpairscore( int nseq1, int nseq2, char **seq1, char **seq2, double *eff1, double *eff2, int penal );
 extern float naiveRpairscore( int nseq1, int nseq2, char **seq1, char **seq2, double *eff1, double *eff2, int penal );
 extern float naiveHpairscore( int nseq1, int nseq2, char **seq1, char **seq2, double *eff1, double *eff2, int penal );
@@ -303,11 +304,11 @@ extern void foldalignedrna( int clus1, int clus2, char **mseq1, char **mseq2, do
 void readmccaskill( FILE *fp, RNApair **pairprob, int length );
 void makegrouprna( RNApair ***group, RNApair ***all, int *memlist );
 void makegrouprnait( RNApair ***group, RNApair ***all, char *pair, int s );
-extern void fixed_musclesupg_float_realloc_nobk_halfmtx( int nseq, float **eff, int ***topol, float **len, Treedep *, int progressout );
+extern void fixed_musclesupg_float_realloc_nobk_halfmtx( int nseq, float **eff, int ***topol, float **len, Treedep *, int progressout, int efffree );
 extern void loadtree( int nseq, int ***topol, float **len, char **name, int *nlen, Treedep * );
 //extern void loadtop( int nseq, float **eff, int ***topol, float **len );
-extern void fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( int nseq, float **eff, int ***topol, float **len, char **name, int *nlen, Treedep * );
-extern void fixed_supg_float_realloc_nobk_halfmtx_treeout_constrained( int nseq, float **eff, int ***topol, float **len, char **name, int *nlen, Treedep *, int ncons, int **constraints );
+extern void fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( int nseq, float **eff, int ***topol, float **len, char **name, int *nlen, Treedep *, int efffree );
+extern void fixed_supg_float_realloc_nobk_halfmtx_treeout_constrained( int nseq, float **eff, int ***topol, float **len, char **name, int *nlen, Treedep *, int ncons, int **constraints, int efffree );
 extern void fixed_musclesupg_double_treeout( int nseq, double **eff, int ***topol, double **len, char **name );
 extern void fixed_supg_double_treeout_constrained( int nseq, double **eff, int ***topol, double **len, char **name, int ncons, int **constraints );
 extern void imp_match_init_strict( float *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, double *eff1kozo, double*eff2kozo, LocalHom ***localhom, int forscore );
@@ -325,12 +326,14 @@ extern void getnumlen_nogap_outallreg_web( FILE *fp, FILE *ofp, int *nlenminpt,
 extern void getnumlen_nogap_outallreg( FILE *fp, int *nlenminpt );
 extern double plainscore( int nseq, char **seq );
 extern void eq2dash( char *s );
+extern void eq2dashmatometehayaku( char **s, int n );
 extern void findnewgaps( int n, int rep, char **seq, int *gaplen );
 extern void findcommongaps( int, char **, int * );
 extern void adjustgapmap( int, int *, char * );
 extern void insertnewgaps( int njob, int *alreadyaligned, char **seq, int *ex1, int *ex2, int *gaplen, int *gapmap, int alloclen, char alg, char gapchar );
 extern void restorecommongaps( int n, char **seq, int *top0, int *top1, int *gaplen, int alloclen, char gapchar );
 extern int samemember( int *mem, int *cand );
+extern int samemembern( int *mem, int *cand, int candn );
 extern int includemember( int *mem, int *cand );
 extern int overlapmember( int *mem1, int *mem2 );
 extern void profilealignment( int n0, int n1, int n2, char **aln0, char **aln1, char **aln2, int alloclen, char alg );
@@ -354,3 +357,5 @@ extern void freeconstants();
 extern void closeFiles();
 extern void FreeCommonIP();
 extern void initglobalvariables();
+extern void makeskiptable( int n, int **skip, char **seq );
+extern int generatesubalignmentstable( int nseq, int ***tablept, int *nsubpt, int *maxmempt, int ***topol, double **len, double threshold );
