@@ -9,13 +9,13 @@
 static TLS int localstop;
 
 #if 1
-static void match_calc( float *match, char **s1, char **s2, int i1, int lgth2 ) 
+static void match_calc_mtx( double **mtx, float *match, char **s1, char **s2, int i1, int lgth2 ) 
 {
-	char tmpc = s1[0][i1];
 	char *seq2 = s2[0];
+	double *doubleptr = mtx[(int)s1[0][i1]];
 
 	while( lgth2-- )
-		*match++ = amino_dis[(int)tmpc][(int)*seq2++];
+		*match++ = doubleptr[(int)*seq2++];
 }
 #else
 static void match_calc( float *match, char **s1, char **s2, int i1, int lgth2 )
@@ -110,7 +110,7 @@ static float gentracking( float *lasthorizontalw, float *lastverticalw,
 }
 
 
-float genL__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *off2pt )
+float genL__align11( double **n_dynamicmtx, char **seq1, char **seq2, int alloclen, int *off1pt, int *off2pt )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
@@ -146,6 +146,7 @@ float genL__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *o
 	static TLS int **intwork;
 	static TLS float **floatwork;
 	static TLS int orlgth1 = 0, orlgth2 = 0;
+	static TLS double **amino_dynamicmtx = NULL; // ??
 	float maxwm;
 	float tbk;
 	int tbki, tbkj;
@@ -186,6 +187,7 @@ float genL__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *o
 
 			FreeFloatMtx( floatwork );
 			FreeIntMtx( intwork );
+			if( amino_dynamicmtx ) FreeDoubleMtx( amino_dynamicmtx ); amino_dynamicmtx = NULL;
 
 		}
 		return( 0.0 );
@@ -229,6 +231,7 @@ float genL__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *o
 
 			FreeFloatMtx( floatwork );
 			FreeIntMtx( intwork );
+			if( amino_dynamicmtx ) FreeDoubleMtx( amino_dynamicmtx ); amino_dynamicmtx = NULL;
 		}
 
 		ll1 = MAX( (int)(1.3*lgth1), orlgth1 ) + 100;
@@ -258,6 +261,8 @@ float genL__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *o
 		floatwork = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 
+		amino_dynamicmtx = AllocateDoubleMtx( 0x80, 0x80 );
+
 #if DEBUG
 		fprintf( stderr, "succeeded\n" );
 #endif
@@ -266,6 +271,8 @@ float genL__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *o
 		orlgth2 = ll2 - 100;
 	}
 
+    for( i=0; i<nalphabets; i++) for( j=0; j<nalphabets; j++ )
+		amino_dynamicmtx[(int)amino[i]][(int)amino[j]] = (double)n_dynamicmtx[i][j];
 
 	mseq1[0] = mseq[0];
 	mseq2[0] = mseq[1];
@@ -310,9 +317,9 @@ float genL__align11( char **seq1, char **seq2, int alloclen, int *off1pt, int *o
 	currentw = w1;
 	previousw = w2;
 
-	match_calc( initverticalw, seq2, seq1, 0, lgth1 );
+	match_calc_mtx( amino_dynamicmtx, initverticalw, seq2, seq1, 0, lgth1 );
 
-	match_calc( currentw, seq1, seq2, 0, lgth2 );
+	match_calc_mtx( amino_dynamicmtx, currentw, seq1, seq2, 0, lgth2 );
 
 
 	lasti = lgth2+1;
@@ -368,7 +375,7 @@ fprintf( stderr, "\n" );
 
 		previousw[0] = initverticalw[i-1];
 
-		match_calc( currentw, seq1, seq2, i, lgth2 );
+		match_calc_mtx( amino_dynamicmtx, currentw, seq1, seq2, i, lgth2 );
 #if DEBUG2
 		fprintf( stderr, "%c   ", seq1[0][i] );
 		fprintf( stderr, "%5.0f ", currentw[0] );
