@@ -22,7 +22,7 @@
 #include <pthread.h>
 #endif
 
-#define VERSION "7.164b"
+#define VERSION "7.205"
 #define SHOWVERSION reporterr( "%s (%s) Version " VERSION " alg=%c, model=%s, amax=%3.1f\n%d thread(s)\n", progName( argv[0] ), (dorp=='d')?"nuc":((nblosum==-2)?"text":"aa"), alg, modelname, specificityconsideration, nthread )
 
 #define FFT_THRESHOLD  80
@@ -323,3 +323,5 @@ extern double specificityconsideration;
 extern int ndistclass, maxdistclass;
 
 extern int gmsg;
+
+extern double sueff_global;
