@@ -265,7 +265,7 @@ static float recallpairfoldalign( char **mseq1, char **mseq2, int m1, int m2, in
 	else
 	{
 		fprintf( stderr, "Calling L__align11\n" );
-		value = L__align11( mseq1, mseq2, alloclen, of1pt, of2pt );
+		value = L__align11( n_dis_consweight_multi, 0.0, mseq1, mseq2, alloclen, of1pt, of2pt );
 	}
 
 //	value = (float)naivepairscore11( *mseq1, *mseq2, penalty ); // nennnotame
@@ -1619,6 +1619,7 @@ void arguments( int argc, char *argv[] )
 	ppenalty_OP = NOTSPECIFIED;
 	ppenalty_ex = NOTSPECIFIED;
 	ppenalty_EX = NOTSPECIFIED;
+	penalty_shift_factor = 1000.0;
 	poffset = NOTSPECIFIED;
 	kimuraR = NOTSPECIFIED;
 	pamN = NOTSPECIFIED;
@@ -1627,6 +1628,7 @@ void arguments( int argc, char *argv[] )
 	fftThreshold = NOTSPECIFIED;
 	RNAppenalty = NOTSPECIFIED;
 	RNApthr = NOTSPECIFIED;
+	specificityconsideration = 0.0;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -1655,6 +1657,10 @@ void arguments( int argc, char *argv[] )
 					ppenalty_EX = (int)( atof( *++argv ) * 1000 - 0.5 );
 					--argc;
 					goto nextoption;
+				case 'Q':
+					penalty_shift_factor = atof( *++argv );
+					--argc;
+					goto nextoption;
 				case 'h':
 					poffset = (int)( atof( *++argv ) * 1000 - 0.5 );
 					--argc;
@@ -1728,6 +1734,11 @@ void arguments( int argc, char *argv[] )
 					fprintf( stderr, "laste = %d\n", laste );
 					--argc;
 					goto nextoption;
+				case 'u':
+					specificityconsideration = (double)myatof( *++argv );
+//					fprintf( stderr, "specificityconsideration = %f\n", specificityconsideration );
+					--argc; 
+					goto nextoption;
 				case 'K': // Hontou ha iranai. disttbfast.c, tbfast.c to awaserutame.
 					break;
 				case 'c':
@@ -1778,7 +1789,7 @@ void arguments( int argc, char *argv[] )
 					alg = 'S';
 					break;
 #endif
-				case 'Q':
+				case 'U':
 					lastonce = 1;
 					break;
 				case 'S':
@@ -1895,6 +1906,20 @@ int countamino( char *s, int end )
 	return( val );
 }
 
+static float score2dist( float pscore, double selfscore1, double selfscore2)
+{
+	float val;
+	double bunbo;
+//	fprintf( stderr, "In score2dist\n" );
+	if( (bunbo=MIN( selfscore1, selfscore2 )) == 0.0 )
+		val = 2.0;
+	else if( bunbo < pscore ) // mondai ari
+		val = 0.0;
+	else
+		val = ( 1.0 - pscore / bunbo ) * 2.0;
+	return( val );
+}
+
 #if enablemultithread
 static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 {
@@ -1902,12 +1927,14 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 	int i, ilim, j, jst;
 	int off1, off2, dum1, dum2, thereisx;
 	int intdum;
-	double bunbo;
 	float pscore = 0.0; // by D.Mathog
 	double *effarr1;
 	double *effarr2;
 	char **mseq1, **mseq2, **distseq1, **distseq2, **dumseq1, **dumseq2;
 	char **aseq;
+	double **dynamicmtx = NULL;
+	float dist;
+	float scoreoffset;
 
 // thread_arg
 	int thread_no = targ->thread_no;
@@ -1938,6 +1965,7 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 	distseq1 = AllocateCharMtx( 1, 0 );
 	distseq2 = AllocateCharMtx( 1, 0 );
 	aseq = AllocateCharMtx( 2, alloclen+10 );
+	if( specificityconsideration > 0.0 ) dynamicmtx = AllocateDoubleMtx( nalphabets, nalphabets );
 
 	if( alg == 'Y' || alg == 'r' ) ilim = njob - nadd;
 	else ilim = njob - 1;
@@ -1968,9 +1996,9 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 				Falign( NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
 				G__align11( NULL, NULL, NULL, 0, 0, 0 ); // 20130603
 				G__align11_noalign( NULL, 0, 0, NULL, NULL, 0 );
-				L__align11( NULL, NULL, 0, NULL, NULL );
-				L__align11_noalign( NULL, NULL );
-				genL__align11( NULL, NULL, 0, NULL, NULL );
+				L__align11( NULL, 0.0, NULL, NULL, 0, NULL, NULL );
+				L__align11_noalign( NULL, NULL, NULL );
+				genL__align11( NULL, NULL, NULL, 0, NULL, NULL );
 				free( effarr1 );
 				free( effarr2 );
 				free( mseq1 );
@@ -1983,6 +2011,7 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 				free( distseq1 );
 				free( distseq2 );
 				FreeCharMtx( aseq  );
+				if( dynamicmtx ) FreeDoubleMtx( dynamicmtx  );
 				return( NULL );
 			}
 		}
@@ -2071,11 +2100,24 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 					{
 						if( store_localhom )
 						{
-							pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
-							if( thereisx ) pscore = L__align11_noalign( distseq1, distseq2 ); // uwagaki
+							pscore = L__align11( n_dis_consweight_multi, 0.0, mseq1, mseq2, alloclen, &off1, &off2 );
+							if( thereisx ) pscore = L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 ); // uwagaki
+#if 1
+							if( specificityconsideration > 0.0 )
+							{
+								dist = score2dist( pscore, selfscore[i], selfscore[j] );
+								if( ( scoreoffset = dist2offset( dist ) ) < 0.0 )
+								{
+									makedynamicmtx( dynamicmtx, n_dis_consweight_multi, 0.5 * dist ); // upgma ni awaseru.
+									strcpy( mseq1[0], seq[i] );
+									strcpy( mseq2[0], seq[j] );
+									L__align11( dynamicmtx, scoreoffset, mseq1, mseq2, alloclen, &off1, &off2 );
+								}
+							}
+#endif
 						}
 						else
-							pscore = L__align11_noalign( distseq1, distseq2 );
+							pscore = L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 );
 					}
 //					pscore = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, distseq1, distseq2, alloclen ); // CHUUI!!!!!!
 					break;
@@ -2084,11 +2126,11 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 					{
 						if( store_localhom )
 						{
-							pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
-							if( thereisx ) pscore = L__align11_noalign( distseq1, distseq2 ); // uwagaki
+							pscore = L__align11( n_dis_consweight_multi, 0.0, mseq1, mseq2, alloclen, &off1, &off2 );
+							if( thereisx ) pscore = L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 ); // uwagaki
 						}
 						else
-							pscore = L__align11_noalign( distseq1, distseq2 );
+							pscore = L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 );
 					}
 					else
 						pscore = 0.0;
@@ -2098,6 +2140,22 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 					{
 						pscore = G__align11( n_dis_consweight_multi, mseq1, mseq2, alloclen, outgap, outgap );
 						if( thereisx ) pscore = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, distseq1, distseq2, alloclen ); // uwagaki
+#if 1
+						if( specificityconsideration > 0.0 )
+						{
+							dist = score2dist( pscore, selfscore[i], selfscore[j] );
+//							dist = score2dist( L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 ), selfscore[i], selfscore[j] ); // 2014/Feb/20
+							if( dist2offset( dist ) < 0.0 )
+							{
+								makedynamicmtx( dynamicmtx, n_dis_consweight_multi, 0.5 * dist ); // upgma ni awaseru.
+								strcpy( mseq1[0], seq[i] );
+								strcpy( mseq2[0], seq[j] );
+								G__align11( dynamicmtx, mseq1, mseq2, alloclen, outgap, outgap );
+				
+							}
+//							pscore = (float)naivepairscore11( *mseq1, *mseq2, 0.0 );
+						}
+#endif
 					}
 					else
 						pscore = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, distseq1, distseq2, alloclen ); // uwagaki
@@ -2105,13 +2163,26 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 					break;
 				case( 'N' ):
 //					pscore = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, mseq1, mseq2, alloclen );
-					pscore = genL__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+					pscore = genL__align11( n_dis_consweight_multi, mseq1, mseq2, alloclen, &off1, &off2 );
 					if( thereisx )
 					{
 						strcpy( dumseq1[0], distseq1[0] );
 						strcpy( dumseq2[0], distseq2[0] );
-						pscore = genL__align11( dumseq1, dumseq2, alloclen, &dum1, &dum2 ); // uwagaki
+						pscore = genL__align11( n_dis_consweight_multi, dumseq1, dumseq2, alloclen, &dum1, &dum2 ); // uwagaki
 					}
+#if 1
+					if( specificityconsideration > 0.0 )
+					{
+						dist = score2dist( pscore, selfscore[i], selfscore[j] );
+						if( dist2offset( dist ) < 0.0 )
+						{
+							makedynamicmtx( dynamicmtx, n_dis_consweight_multi, 0.5 * dist ); // upgma ni awaseru.
+							strcpy( mseq1[0], seq[i] );
+							strcpy( mseq2[0], seq[j] );
+							genL__align11( dynamicmtx, mseq1, mseq2, alloclen, &off1, &off2 );
+						}
+					}
+#endif
 					break;
 				case( 't' ):
 					off1 = off2 = 0;
@@ -2173,13 +2244,7 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 				else if( alg != 'S' && alg != 'V' )
 					putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
 			}
-
-			if( (bunbo=MIN( selfscore[i], selfscore[j] )) == 0.0 )
-				pscore = 2.0;
-			else if( bunbo < pscore ) // mondai ari
-				pscore = 0.0;
-			else
-				pscore = ( 1.0 - pscore / bunbo ) * 2.0;
+			pscore = score2dist( pscore, selfscore[i], selfscore[j] );
 		}
 		else
 		{
@@ -2232,10 +2297,11 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 	char *hat2file = "hat2";
 	LocalHom **localhomtable = NULL, *tmpptr;
 	int intdum;
-	double bunbo;
 	char ***bpp = NULL; // mxscarna no toki dake
 	char **distseq1, **distseq2;
 	char **dumseq1, **dumseq2;
+	float dist;
+	float scoreoffset;
 
 	if( store_localhom )
 	{
@@ -2427,6 +2493,9 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 	else
 #endif
 	{
+		double **dynamicmtx = NULL;
+		if( specificityconsideration > 0.0 ) dynamicmtx = AllocateDoubleMtx( nalphabets, nalphabets );
+
 		if( alg == 'Y' || alg == 'r' ) ilim = njob - nadd;
 		else ilim = njob - 1;
 		for( i=0; i<ilim; i++ ) 
@@ -2499,6 +2568,21 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 							{
 								pscore = G__align11( n_dis_consweight_multi, mseq1, mseq2, alloclen, outgap, outgap );
 								if( thereisx ) pscore = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, distseq1, distseq2, alloclen ); // uwagaki
+#if 1
+								if( specificityconsideration > 0.0 )
+								{
+									dist = score2dist( pscore, selfscore[i], selfscore[j] );
+//									dist = score2dist( L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 ), selfscore[i], selfscore[j] ); // 2014/Feb/20
+									if( dist2offset( dist ) < 0.0 )
+									{
+										makedynamicmtx( dynamicmtx, n_dis_consweight_multi, 0.5 * dist ); // upgma ni awaseru.
+										strcpy( mseq1[0], seq[i] );
+										strcpy( mseq2[0], seq[j] );
+										G__align11( dynamicmtx, mseq1, mseq2, alloclen, outgap, outgap );
+									}
+//									pscore = (float)naivepairscore11( *mseq1, *mseq2, 0.0 );
+								}
+#endif
 							}
 							else
 								pscore = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, distseq1, distseq2, alloclen ); // uwagaki
@@ -2506,13 +2590,27 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 							break;
 						case( 'N' ):
 //							pscore = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, mseq1, mseq2, alloclen );
-							pscore = genL__align11( mseq1, mseq2, alloclen, &off1, &off2 );
+							pscore = genL__align11( n_dis_consweight_multi, mseq1, mseq2, alloclen, &off1, &off2 );
 							if( thereisx )
 							{
 								strcpy( dumseq1[0], distseq1[0] );
 								strcpy( dumseq2[0], distseq2[0] );
-								pscore = genL__align11( dumseq1, dumseq2, alloclen, &dum1, &dum2 ); // uwagaki
+								pscore = genL__align11( n_dis_consweight_multi, dumseq1, dumseq2, alloclen, &dum1, &dum2 ); // uwagaki
 							}
+#if 1
+							if( specificityconsideration > 0.0 )
+							{
+//								fprintf( stderr, "dist = %f\n", score2dist( pscore, selfscore[i], selfscore[j] ) );
+								dist = score2dist( pscore, selfscore[i], selfscore[j] );
+								if( dist2offset( dist ) < 0.0 )
+								{
+									makedynamicmtx( dynamicmtx, n_dis_consweight_multi, 0.5 * dist ); // upgma ni awaseru.
+									strcpy( mseq1[0], seq[i] );
+									strcpy( mseq2[0], seq[j] );
+									genL__align11( dynamicmtx, mseq1, mseq2, alloclen, &off1, &off2 );
+								}
+							}
+#endif
 							break;
 						case( 'R' ):
 							if( nadd && njob-nadd <= j && njob-nadd <= i ) // new sequence doushi ha mushi
@@ -2533,11 +2631,24 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 							{
 								if( store_localhom )
 								{
-									pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 ); // all pair
-									if( thereisx ) pscore = L__align11_noalign( distseq1, distseq2 ); // all pair
+									pscore = L__align11( n_dis_consweight_multi, 0.0, mseq1, mseq2, alloclen, &off1, &off2 ); // all pair
+									if( thereisx ) pscore = L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 ); // all pair
+#if 1
+									if( specificityconsideration > 0.0 )
+									{
+										dist = score2dist( pscore, selfscore[i], selfscore[j] );
+										if( ( scoreoffset = dist2offset( dist ) ) < 0.0 )
+										{
+											makedynamicmtx( dynamicmtx, n_dis_consweight_multi, 0.5 * dist ); // upgma ni awaseru.
+											strcpy( mseq1[0], seq[i] );
+											strcpy( mseq2[0], seq[j] );
+											L__align11( dynamicmtx, scoreoffset, mseq1, mseq2, alloclen, &off1, &off2 );
+										}
+									}
+#endif
 								}
 								else
-									pscore = L__align11_noalign( distseq1, distseq2 ); // all pair
+									pscore = L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 ); // all pair
 							}
 //							pscore = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, distseq1, distseq2, alloclen ); // CHUUI!!!!!!
 							break;
@@ -2546,11 +2657,11 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 							{
 								if( store_localhom )
 								{
-									pscore = L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
-									if( thereisx ) pscore = L__align11_noalign( distseq1, distseq2 ); // uwagaki
+									pscore = L__align11( n_dis_consweight_multi, 0.0, mseq1, mseq2, alloclen, &off1, &off2 );
+									if( thereisx ) pscore = L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 ); // uwagaki
 								}
 								else
-									pscore = L__align11_noalign( distseq1, distseq2 );
+									pscore = L__align11_noalign( n_dis_consweight_multi, distseq1, distseq2 );
 							}
 							else
 								pscore = 0.0;
@@ -2611,12 +2722,7 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 							putlocalhom2( mseq1[0], mseq2[0], localhomtable[i]+j, off1, off2, (int)pscore, strlen( mseq1[0] ) );
 					}
 
-					if( (bunbo=MIN( selfscore[i], selfscore[j] )) == 0.0 )
-						pscore = 2.0;
-					else if( bunbo < pscore ) // mondai ari
-						pscore = 0.0;
-					else
-						pscore = ( 1.0 - pscore / bunbo ) * 2.0;
+					pscore = score2dist( pscore, selfscore[i], selfscore[j] );
 				}
 				else
 				{
@@ -2643,6 +2749,7 @@ static void pairalign( char **name, int *nlen, char **seq, char **aseq, char **d
 				}
 			}
 		}
+		if( dynamicmtx ) FreeDoubleMtx( dynamicmtx );
 	}
 
 
