@@ -569,6 +569,7 @@ void arguments( int argc, char *argv[] )
 	tbrweight = 3;
 	checkC = 0;
 	treemethod = 'X';
+	sueff_global = 0.1;
 	contin = 0;
 	scoremtx = 1;
 	kobetsubunkatsu = 0;
@@ -747,8 +748,11 @@ void arguments( int argc, char *argv[] )
 					tbutree = 0;
 					break;
 				case 'X':
-					treemethod = 'X'; // mix
-					break;
+					treemethod = 'X'; // tsukawareteiru ????
+					sueff_global = atof( *++argv );
+					fprintf( stderr, "sueff_global = %f\n", sueff_global );
+					--argc;
+					goto nextoption;
 				case 'E':
 					treemethod = 'E'; // upg (average)
 					break;
@@ -2286,7 +2290,7 @@ exit( 1 );
 		{
 			fprintf( stderr, "upgma       " );
 //			veryfastsupg_float_realloc_nobk_halfmtx( nyuko, yukomtx, topol, len );
-			fixed_musclesupg_float_realloc_nobk_halfmtx( nyuko, yukomtx, topol, len, NULL, 1 );
+			fixed_musclesupg_float_realloc_nobk_halfmtx( nyuko, yukomtx, topol, len, NULL, 1, 1 );
 			fprintf( stderr, "\r                      \r" );
 		}
 		else
