@@ -28,6 +28,7 @@ typedef struct _distancematrixthread_arg
 	float *selfscore;
 	float **iscore;
 	char **seq;
+	int **skiptable;
 	Jobtable *jobpospt;
 	pthread_mutex_t *mutex;
 } distancematrixthread_arg_t;
@@ -96,6 +97,7 @@ void arguments( int argc, char *argv[] )
 	tbrweight = 3;
 	checkC = 0;
 	treemethod = 'X';
+	sueff_global = 0.1;
 	contin = 0;
 	scoremtx = 1;
 	kobetsubunkatsu = 0;
@@ -280,7 +282,10 @@ void arguments( int argc, char *argv[] )
 #endif
 				case 'X':
 					treemethod = 'X';
-					break;
+					sueff_global = atof( *++argv );
+					fprintf( stderr, "sueff_global = %f\n", sueff_global );
+					--argc;
+					goto nextoption;
 				case 'E':
 					treemethod = 'E';
 					break;
@@ -440,6 +445,7 @@ static void *distancematrixthread( void *arg )
 	float *selfscore = targ->selfscore;
 	float **iscore = targ->iscore;
 	char **seq = targ->seq;
+	int **skiptable = targ->skiptable;
 	Jobtable *jobpospt = targ->jobpospt;
 
 	float ssi, ssj, bunbo;
@@ -473,7 +479,8 @@ static void *distancematrixthread( void *arg )
 		if( bunbo == 0.0 )
 			iscore[i][j-i] = 2.0; // 2013/Oct/17
 		else
-			iscore[i][j-i] = ( 1.0 - naivepairscore11( seq[i], seq[j], penalty_dist ) / bunbo ) * 2.0; // 2013/Oct/17
+//			iscore[i][j-i] = ( 1.0 - naivepairscore11( seq[i], seq[j], penalty_dist ) / bunbo ) * 2.0; // 2013/Oct/17
+			iscore[i][j-i] = ( 1.0 - naivepairscorefast( seq[i], seq[j], skiptable[i], skiptable[j], penalty_dist ) / bunbo ) * 2.0; // 2014/Aug/15 fast
 	}
 }
 
@@ -1171,7 +1178,14 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen, '-' );
 			findnewgaps( clus1, 0, mseq1, gaplen );
 			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, gapmap, *alloclen, alg, '-' );
+#if 0
 			for( i=0; i<njob; i++ ) eq2dash( aseq[i] );
+			for( i=0; i<clus1; i++ ) eq2dash( mseq1[i] );
+			for( i=0; i<clus2; i++ ) eq2dash( mseq2[i] );
+#else
+			eq2dashmatometehayaku( mseq1, clus1 );
+			eq2dashmatometehayaku( mseq2, clus2 );
+#endif
 			for( i=0; (m=topol[l][1][i])>-1; i++ ) alreadyaligned[m] = 1;
 		}
 
@@ -1278,6 +1292,7 @@ int main( int argc, char *argv[] )
 	static char **mseq1, **mseq2;
 	static char **bseq;
 	static float **iscore, **iscore_kozo;
+	int **skiptable;
 	static double *eff, *eff_kozo, *eff_kozo_mapped = NULL;
 	int i, j, ien, ik, jk;
 	static int ***topol, ***topol_kozo;
@@ -1480,10 +1495,13 @@ int main( int argc, char *argv[] )
 	
 			fprintf( stderr, "Making a distance matrix .. \n" );
 			fflush( stderr );
+			skiptable = AllocateIntMtx( njob, 0 );
+			makeskiptable( njob, skiptable, seq ); // allocate suru.
 			ien = njob-1;
 			for( i=0; i<njob; i++ ) 
 			{
-				selfscore[i] = naivepairscore11( seq[i], seq[i], penalty_dist );
+//				selfscore[i] = naivepairscore11( seq[i], seq[i], penalty_dist );
+				selfscore[i] = naivepairscorefast( seq[i], seq[i], skiptable[i], skiptable[i], penalty_dist );
 //				fprintf( stderr, "penalty = %d\n", penalty );
 //				fprintf( stderr, "penalty_dist = %d\n", penalty_dist );
 			}
@@ -1509,6 +1527,7 @@ int main( int argc, char *argv[] )
 					targ[i].selfscore = selfscore;
 					targ[i].iscore = iscore;
 					targ[i].seq = seq;
+					targ[i].skiptable = skiptable;
 					targ[i].jobpospt = &jobpos;
 					targ[i].mutex = &mutex;
 
@@ -1542,7 +1561,9 @@ int main( int argc, char *argv[] )
 							iscore[i][j-i] = 2.0; // 2013/Oct/17 2bai
 						else
 //							iscore[i][j-i] = 1.0 - naivepairscore11( seq[i], seq[j], penalty_dist ) / MIN( selfscore[i], selfscore[j] );
-							iscore[i][j-i] = ( 1.0 - naivepairscore11( seq[i], seq[j], penalty_dist ) / bunbo ) * 2.0; // 2013/Oct/17 2bai
+//							iscore[i][j-i] = ( 1.0 - naivepairscore11( seq[i], seq[j], penalty_dist ) / bunbo ) * 2.0; // 2013/Oct/17 2bai
+							iscore[i][j-i] = ( 1.0 - naivepairscorefast( seq[i], seq[j], skiptable[i], skiptable[j], penalty_dist ) / bunbo ) * 2.0; // 2014/Aug/15 fast
+//exit( 1 );
 		
 #if 0
 						fprintf( stderr, "### ssj = %f\n", ssj );
@@ -1554,6 +1575,7 @@ int main( int argc, char *argv[] )
 				}
 			}
 			fprintf( stderr, "\ndone.\n\n" );
+			FreeIntMtx( skiptable );
 			fflush( stderr );
 		}
 		else
@@ -1623,15 +1645,15 @@ int main( int argc, char *argv[] )
 		}
 		else if( subalignment ) // merge error no tame
 		{
-			fixed_supg_float_realloc_nobk_halfmtx_treeout_constrained( njob, iscore, topol, len, name, nlen, dep, nsubalignments, subtable );
+			fixed_supg_float_realloc_nobk_halfmtx_treeout_constrained( njob, iscore, topol, len, name, nlen, dep, nsubalignments, subtable, 1 );
 		}
 		else if( treeout ) // merge error no tame
 		{
-			fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( njob, iscore, topol, len, name, nlen, dep );
+			fixed_musclesupg_float_realloc_nobk_halfmtx_treeout( njob, iscore, topol, len, name, nlen, dep, 1 );
 		}
 		else
 		{
-			fixed_musclesupg_float_realloc_nobk_halfmtx( njob, iscore, topol, len, dep, 1 );
+			fixed_musclesupg_float_realloc_nobk_halfmtx( njob, iscore, topol, len, dep, 1, 1 );
 		}
 //		else 
 //			ErrorExit( "Incorrect tree\n" );
@@ -1641,7 +1663,7 @@ int main( int argc, char *argv[] )
 //			for( i=0; i<nkozo-1; i++ )
 //				for( j=i+1; j<nkozo; j++ )
 //					fprintf( stderr, "iscore_kozo[%d][%d] =~ %f\n", i, j, iscore_kozo[i][j-i] );
-			fixed_musclesupg_float_realloc_nobk_halfmtx( nkozo, iscore_kozo, topol_kozo, len_kozo, NULL, 1 );
+			fixed_musclesupg_float_realloc_nobk_halfmtx( nkozo, iscore_kozo, topol_kozo, len_kozo, NULL, 1, 1 );
 		}
 		fprintf( stderr, "\ndone.\n\n" );
 		fflush( stderr );
@@ -1771,7 +1793,7 @@ int main( int argc, char *argv[] )
 					mergeoralign[i] = '1';
 					foundthebranch = 1;
 				}
-				else if( samemember( topol[i][1], addmem ) )
+				else if( samemember( topol[i][1], addmem ) ) // samemembern ni henkou kanou
 				{
 					mergeoralign[i] = '2';
 					foundthebranch = 1;
@@ -1803,13 +1825,13 @@ int main( int argc, char *argv[] )
 				addmem[1] = -1;
 				for( i=0; i<njob-1; i++ )
 				{
-					if( samemember( topol[i][0], addmem ) ) // arieru
+					if( samemembern( topol[i][0], addmem, 1 ) ) // arieru
 					{
 //						fprintf( stderr, "HIT!\n" );
 						if( mergeoralign[i] != 'n' ) mergeoralign[i] = 'w';
 						else mergeoralign[i] = '1';
 					}
-					else if( samemember( topol[i][1], addmem ) )
+					else if( samemembern( topol[i][1], addmem, 1 ) )
 					{
 //						fprintf( stderr, "HIT!\n" );
 						if( mergeoralign[i] != 'n' ) mergeoralign[i] = 'w';
