@@ -732,9 +732,9 @@ void nodeFromABranch( int nseq, int *result, int **pairwisenode, int ***topol, d
 					
 
 				
-void OneClusterAndTheOther_fast( int locnjob, int *memlist1, int *memlist2, int *s1, int *s2, char *pair, int ***topol, int step, int branch )
+void OneClusterAndTheOther_fast( int locnjob, int *memlist1, int *memlist2, int *s1, int *s2, char *pair, int ***topol, int step, int branch, double **smalldistmtx, double **distmtx )
 {
-	int i, k;
+	int i, k, j;
 	int r1;
 //	char *pair;
 
@@ -759,6 +759,16 @@ void OneClusterAndTheOther_fast( int locnjob, int *memlist1, int *memlist2, int
 
 	*s1 = memlist1[0];
 	*s2 = memlist2[0];
+
+	if( smalldistmtx )
+	{
+		int im, jm;
+		for( i=0; (im=memlist1[i])!=-1; i++ ) for( j=0; (jm=memlist2[j])!=-1; j++ )
+		{
+			smalldistmtx[i][j] = distmtx[MIN(im,jm)][MAX(im,jm)];
+//			fprintf( stderr, "#### %d-%d, %f\n", im, jm, smalldistmtx[i][j] );
+		}
+	}
 //	free( pair );
 }
 		
