@@ -335,7 +335,7 @@ static void *athread( void *arg )
 	int **skipthisbranch = targ->skipthisbranch;
 	double **distmtx = targ->distmtx;
 
-	int i, k, l, ii, j;
+	int i, k, l, ii;
 	float gain;
 	int iterate;
 	int **memlist;
@@ -356,7 +356,7 @@ static void *athread( void *arg )
 	float tscore, mscore, oimpmatch, impmatch;
 	int identity;
 	double tmpdouble;
-	float naivescore0 = 0, naivescore1;
+//	float naivescore0 = 0, naivescore1;
 	double *effarrforlocalhom;
 	float *tscorehistory;
 	int intdum;
@@ -730,6 +730,10 @@ static void *athread( void *arg )
 			OneClusterAndTheOther_fast( locnjob, memlist[0], memlist[1], &s1, &s2, pairbuf, topol, l, k, smalldistmtx, distmtx );
 
 
+//			reporterr( "\n\n\n\n##### memlist[0][0], memlist[1][0] = %d, %d\n", memlist[0][0]+1, memlist[1][0]+1 );
+
+
+
 #if 0
 			fprintf( stderr, "STEP%d-%d\n", l, k );
 			for( i=0; i<locnjob; i++ ) 
@@ -1227,8 +1231,8 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 	int identity;
 	int converged;
 	int oscillating;
-	float naivescore0 = 0.0; // by D.Mathog, a guess
-	float naivescore1;
+//	float naivescore0 = 0.0; // by D.Mathog, a guess
+//	float naivescore1;
 #if 0
 	char pair[njob][njob];
 #else
@@ -1626,6 +1630,14 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 
 //					for( i=0; i<2; i++ ) for( j=0; j<locnjob; j++ ) pair[i][j] = 0;
 					OneClusterAndTheOther_fast( locnjob, memlist[0], memlist[1], &s1, &s2, pairbuf, topol, l, k, smalldistmtx, distmtx );
+
+
+
+//					reporterr( "\n\n##### memlist[0][0] = %d\n", memlist[0][0]+1 );
+//					reporterr( "##### memlist[1][0] = %d\n\n", memlist[1][0]+1 );
+
+
+
 	#if 0 // IRANAI!!!!
 					fprintf( stderr, "STEP%d-%d\n", l, k );
 					for( i=0; i<2; i++ ) 
